package fastly

import (
	"bytes"
	"testing"

	"github.com/ajg/form"
)

func TestClient_Settings(t *testing.T) {
}

// Tests if we can update a default_ttl to 0 as reported in issue #20
func TestClient_UpdateSettingsInput_default_ttl(t *testing.T) {
	t.Parallel()
	s := UpdateSettingsInput{Service: "foo", Version: 1, DefaultTTL: 0}
	buf := new(bytes.Buffer)
	form.NewEncoder(buf).KeepZeros(true).DelimitWith('|').Encode(s)
	if buf.String() != "Service=foo&Version=1&general.default_ttl=0" {
		t.Errorf("Update request should contain a default_ttl. Got: %s", buf.String())
	}
}

func TestClient_GetSettings_validation(t *testing.T) {
	var err error
	_, err = testClient.GetSettings(&GetSettingsInput{
		Service: "",
	})
	if err != ErrMissingService {
		t.Errorf("bad error: %s", err)
	}

	_, err = testClient.GetSettings(&GetSettingsInput{
		Service: "foo",
		Version: 0,
	})
	if err != ErrMissingVersion {
		t.Errorf("bad error: %s", err)
	}
}

func TestClient_UpdateSettings_validation(t *testing.T) {
	var err error
	_, err = testClient.UpdateSettings(&UpdateSettingsInput{
		Service: "",
	})
	if err != ErrMissingService {
		t.Errorf("bad error: %s", err)
	}

	_, err = testClient.UpdateSettings(&UpdateSettingsInput{
		Service: "foo",
		Version: 0,
	})
	if err != ErrMissingVersion {
		t.Errorf("bad error: %s", err)
	}
}
